//+------------------------------------------------------------------+ 
//|   Statistical Analysis using Bollinger Bands - for FX v1.0       |
//|                                                                  | 
//|   Copyright  2014 / airquest@hotmail.com                        |
//|                                                                  |
//|   http://smart-trading.world-record.ch                           |  
//+------------------------------------------------------------------+ 

#property copyright   "Copyright  2014 // airquest // smart-trading.world-record.ch"
#property link        "http://smart-trading.world-record.ch"
#property description "Statistical Analysis using Bollinger Bands - for FX"
#property version     "1.0"

//+------------------------------------------------------------------+
//| Buffers                                                          |
//+------------------------------------------------------------------+

#property indicator_chart_window
#property indicator_buffers 16

//+------------------------------------------------------------------+
//| Enumerations                                                     |
//+------------------------------------------------------------------+

enum ENUM_PRICES
{
   close          = 0,  // Close
   high_low       = 1,  // High-Low
   median         = 2,  // Median
   typical        = 3,  // Typical
   weighted_close = 4   // Weighted Close
};

//+------------------------------------------------------------------+
//| External variables                                               |
//+------------------------------------------------------------------+

extern int               BandsPeriod   = 20;
input ENUM_MA_METHOD     BandsMethod   = MODE_SMA;
input ENUM_APPLIED_PRICE BandsPrice    = PRICE_CLOSE;
extern double            Deviation     = 2.0;
extern int               BandsShift    =  0;
input ENUM_PRICES        PriceForBreak = high_low;

extern int               CCIPeriod     = 14;
input ENUM_APPLIED_PRICE CCIPrice      = PRICE_TYPICAL;
extern int               CCIOBOSLevel  = 150;

extern string            _Optimization = "=============== Optimization ===============";
extern bool              Optimize      = True;
extern double            StopLoss      = 300;
extern double            TakeProfit    = 500;
extern double            Spread        = 10;
extern double            Lots          = 1.0;

extern string            _Others                  = "---------------------- Others Settings ----------------------";
extern bool              AllowMultipleOpenTrades  = True;
extern int               BarsToCount              = 1000;
extern bool              UseAllBars               = True;
extern bool              RunOnlyOnce              = True;
extern bool              PlaySoundAtTheEnd        = True;
extern string            Sound                    = "alert2.wav";

//+------------------------------------------------------------------+
//| Internal variables                                               |
//+------------------------------------------------------------------+

double MovingBuffer[];
double UpperBuffer[];
double LowerBuffer[];

double Loss[];
double Win[];
double Balance[];

double SL[];
double TP[];
double RecSL[];
double RecTP[];
double Direction[];
double FirstLossShift[];
double FirstWinShift[];

double PriceUp[];
double PriceDn[];

double Osc[];

static double TotalTrades,TotalWins,WinRate,TotalBalance;
static double OptiTotalTrades,OptiTotalWins,OptiWinRate,OptiTotalBalance;

//+------------------------------------------------------------------+
//| Initialization function                                          |
//+------------------------------------------------------------------+

int init()
{
   IndicatorDigits(Digits);

   IndicatorBuffers(16);

   SetIndexBuffer(0,MovingBuffer);  SetIndexLabel(0,"MA");
   SetIndexBuffer(1,UpperBuffer);   SetIndexLabel(1,"Upper Band");
   SetIndexBuffer(2,LowerBuffer);   SetIndexLabel(2,"Lower Band");

   SetIndexDrawBegin(0,BandsPeriod+BandsShift);    SetIndexShift(0,BandsShift);
   SetIndexDrawBegin(1,BandsPeriod+BandsShift);    SetIndexShift(1,BandsShift);
   SetIndexDrawBegin(2,BandsPeriod+BandsShift);    SetIndexShift(2,BandsShift);

   SetIndexBuffer(3,Loss);             SetIndexLabel(3,"Loss");
   SetIndexBuffer(4,Win);              SetIndexLabel(4,"Win");
   SetIndexBuffer(5,Balance);          SetIndexLabel(5,"Balance");
   SetIndexBuffer(6,SL);               SetIndexLabel(6,"Stop Loss");
   SetIndexBuffer(7,TP);               SetIndexLabel(7,"Take Profit");
   SetIndexBuffer(8,RecSL);            SetIndexLabel(8,"Rec SL");
   SetIndexBuffer(9,RecTP);            SetIndexLabel(9,"Rec TP");
   SetIndexBuffer(10,Direction);       SetIndexLabel(10,"Direction");
   SetIndexBuffer(11,FirstLossShift);  SetIndexLabel(11,"First Loss Shift");
   SetIndexBuffer(12,FirstWinShift);   SetIndexLabel(12,"First Win Shift");

   SetIndexBuffer(13,PriceUp);         SetIndexLabel(13,"Price High");
   SetIndexBuffer(14,PriceDn);         SetIndexLabel(14,"Price Low");  

   SetIndexBuffer(15,Osc);             SetIndexLabel(15,"Oscillator");

   for(int i=0; i<16; i++) SetIndexStyle(i,DRAW_NONE);

   return(0);
}

//+------------------------------------------------------------------+
//| Deinitialization function                                        |
//+------------------------------------------------------------------+

int deinit()
{
   DeleteLabels();
   return(0);
}

//+------------------------------------------------------------------+
//| Main calculation                                                 |
//+------------------------------------------------------------------+

int start()
{
//+------------------------------------------------------------------+ Avoid refreshing

   if(RunOnlyOnce) { static int CanRun = 0; if(CanRun == 0) CanRun = 1; else return(-1); }

//+------------------------------------------------------------------+ Setting limit

   int counted_bars=IndicatorCounted();
   if (counted_bars<0) return(-1);
   if (counted_bars>0) counted_bars--;
   int i,limit = MathMin(Bars-counted_bars,MathMin(Bars-1,MathMin(Bars-BandsPeriod,Bars-CCIPeriod)));

//+------------------------------------------------------------------+ Setting prices buffers

   for(i=limit; i>=0; i--)
   {
      switch(PriceForBreak)
      {
         case 0: { PriceUp[i] = Close[i];                            PriceDn[i] = Close[i];                          } break;
         case 1: { PriceUp[i] = High[i];                             PriceDn[i] = Low[i];                            } break;
         case 2: { PriceUp[i] = (High[i] + Low[i])/2;                PriceDn[i] = (High[i] + Low[i])/2;              } break;
         case 3: { PriceUp[i] = (High[i] + Low[i] + Close[i])/3;     PriceDn[i] = (High[i] + Low[i] + Close[i])/3;   } break;
         case 4: { PriceUp[i] = (High[i] + Low[i] + 2*Close[i])/4;   PriceDn[i] = (High[i] + Low[i] + 2*Close[i])/4; } break;
      }
   }

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+       CALCULATION 1: Results coming from the Optimiziation double function

   if(Optimize)
   {
      OptiTotalTrades  = Optimization(limit,BarsToCount,BandsPeriod,BandsPrice,BandsMethod,Deviation,CCIPeriod,CCIPrice,CCIOBOSLevel,
                                            StopLoss,TakeProfit,PriceUp,PriceDn,"TotalTrades");
      OptiTotalWins    = Optimization(limit,BarsToCount,BandsPeriod,BandsPrice,BandsMethod,Deviation,CCIPeriod,CCIPrice,CCIOBOSLevel,
                                            StopLoss,TakeProfit,PriceUp,PriceDn,"TotalWins");
      OptiWinRate      = Optimization(limit,BarsToCount,BandsPeriod,BandsPrice,BandsMethod,Deviation,CCIPeriod,CCIPrice,CCIOBOSLevel,
                                            StopLoss,TakeProfit,PriceUp,PriceDn,"WinRate");
      OptiTotalBalance = Optimization(limit,BarsToCount,BandsPeriod,BandsPrice,BandsMethod,Deviation,CCIPeriod,CCIPrice,CCIOBOSLevel,
                                            StopLoss,TakeProfit,PriceUp,PriceDn,"TotalBalance");
   }

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+       CALCULATION 2: Results coming from the Start function

   for(i=limit; i>=0; i--)
   {

//+------------------------------------------------------------------+ Setting buffers

      Loss[i]           = 0;
      Win[i]            = 0;
      Balance[i]        = 0;
      SL[i]             = 0;
      TP[i]             = 0;
      RecTP[i]          = 0;
      RecSL[i]          = 0;
      Direction[i]      = 0;
      FirstLossShift[i] = 0;
      FirstWinShift[i]  = 0;

      double TV = MarketInfo(NULL,MODE_TICKVALUE);

      MovingBuffer[i]   = iMA    (NULL,0,BandsPeriod,0,BandsMethod,BandsPrice,i);
      double deviation  = iStdDev(NULL,0,BandsPeriod,0,MODE_SMA,BandsPrice,i);
      UpperBuffer[i]    = MovingBuffer[i] + deviation * Deviation;
      LowerBuffer[i]    = MovingBuffer[i] - deviation * Deviation;
      Osc[i]            = iCCI(NULL,0,CCIPeriod,CCIPrice,i);

//+------------------------------------------------------------------+ Finding signals

      if(!UseAllBars && BarsToCount < limit) int limitBars = BarsToCount;
      else                                       limitBars = limit;

      if(i <= limitBars)
      {
         bool dontTrade = PriceUp[i] > UpperBuffer[i] && PriceDn[i] < LowerBuffer[i];

         if     (!dontTrade && Osc[i] >  CCIOBOSLevel && PriceUp[i] > UpperBuffer[i])    // Sell signals
         {
            SL[i] = iOpen(NULL,0,i-1) + (StopLoss   / MathPow(10,Digits));
            TP[i] = iOpen(NULL,0,i-1) - (TakeProfit / MathPow(10,Digits));
            Direction[i] = -1;
         }

         else if(!dontTrade && Osc[i] < -CCIOBOSLevel && PriceDn[i] < LowerBuffer[i])    // Buy signals
         {
            SL[i] = iOpen(NULL,0,i-1) - (StopLoss   / MathPow(10,Digits));
            TP[i] = iOpen(NULL,0,i-1) + (TakeProfit / MathPow(10,Digits));
            Direction[i] =  1;
         }

//+------------------------------------------------------------------+ Finding if trade is a winner or loser

         if     (Direction[i] ==  1)
         {
            FirstLossShift[i] = CheckFirstLoserOrWinner(i,True, True, SL[i],TP[i]);
            FirstWinShift[i]  = CheckFirstLoserOrWinner(i,True, False,SL[i],TP[i]);
         }

         else if(Direction[i] == -1)
         {
            FirstLossShift[i] = CheckFirstLoserOrWinner(i,False,True, SL[i],TP[i]);
            FirstWinShift[i]  = CheckFirstLoserOrWinner(i,False,False,SL[i],TP[i]);
         }

         if     (FirstWinShift[i] > FirstLossShift[i])
         {
            Win[i] = 1; Loss[i] = 0; Balance[i] =  TakeProfit*Lots*TV;
         }

         else if(FirstWinShift[i] < FirstLossShift[i])
         {
            Win[i] = 0; Loss[i] = 1; Balance[i] = -StopLoss  *Lots*TV;
         }

//+------------------------------------------------------------------+ Deleting trades when another trade was already open

         if(!AllowMultipleOpenTrades)
         {
            if(TP[i] > 0) RecTP[i] = TP[i]; else if (RecTP[i+1] > 0) RecTP[i] = RecTP[i+1];
            if(SL[i] > 0) RecSL[i] = SL[i]; else if (RecSL[i+1] > 0) RecSL[i] = RecSL[i+1];
   
            if((High[i] < RecTP[i+1] && Low[i] > RecSL[i+1]) || (High[i] < RecSL[i+1] && Low[i] > RecTP[i+1]))
            {
               RecTP[i] = RecTP[i+1];
               RecSL[i] = RecSL[i+1];
            }

            if(TP[i] != RecTP[i] || SL[i] != RecSL[i]) 
            {
               Win[i] = 0; Loss[i] = 0; Balance[i] = 0;
            }
         }

//+------------------------------------------------------------------+ Totals

         TotalWins    += Win[i];
         TotalTrades  += (Win[i] + Loss[i]);
         TotalBalance += Balance[i];

         if(TotalTrades == 0) WinRate = 0;
         else                 WinRate = (TotalWins * 100) / TotalTrades;
      }
   }

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+ Labels

   string Line1 = "Results coming from the Start function:";
   string Line2 = "Trades: "+TotalTrades+" / Wins: "+TotalWins+" / Win%: "+DoubleToString(WinRate,2)+"% / Balance: "+TotalBalance;

   color couleur = White;
   int   corner  = 0;
   int   x = 10;
   int   y = 40;

   DrawLabels(1,corner,x,y,   Line1,couleur);
   DrawLabels(2,corner,x,y+10,Line2,couleur);

   string Line5 = "Results imported from the Opti function:";
   string Line6 = "Trades: "+OptiTotalTrades+" / Wins: "+OptiTotalWins+" / Win%: "+DoubleToString(OptiWinRate,2)+"% / Balance: "+OptiTotalBalance;

   DrawLabels(5,corner,x,y+60,Line5,couleur);
   DrawLabels(6,corner,x,y+70,Line6,couleur);

   if(PlaySoundAtTheEnd) PlaySound(Sound);

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+

   return(0);
}

//+---------------------------------------------------------------------------------------+
//| Functions                                                                             |
//+---------------------------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Optimization                                                     |
//+------------------------------------------------------------------+

double Optimization(int limit, int barsLimit, int bbPeriod, int bbPrice, int bbMethod, double bbDeviation, int cciPeriod, int cciPrice, int cciLevel,
                    double stopLoss, double takeProfit, double& priceUp[], double& priceDn[], string whichResult)
{
   int    i;
   int    direction      = 0;
   int    firstLossShift = 0;
   int    firstWinShift  = 0;
   double sl             = 0;
   double tp             = 0;
   double loss           = 0;
   double win            = 0;
   double balance        = 0;
   double totalWins      = 0;
   double totalTrades    = 0;
   double totalBalance   = 0;
   double result         = 0;

   double TV = MarketInfo(NULL,MODE_TICKVALUE);

   double Ma,Dev,Up,Dn,cci;
   double recTP[],recSL[];

   if(ArrayRange(recSL,0) != limit) ArrayResize(recSL,limit);
   if(ArrayRange(recTP,0) != limit) ArrayResize(recTP,limit);

   for(i=limit; i>=0; i--)
   {

//+------------------------------------------------------------------+ Setting buffers

      Ma  = iMA    (NULL,0,bbPeriod,0,bbMethod,bbPrice,i);
      Dev = iStdDev(NULL,0,bbPeriod,0,MODE_SMA,bbPrice,i);
      Up  = Ma + Dev * bbDeviation;
      Dn  = Ma - Dev * bbDeviation;
      cci = iCCI(NULL,0,cciPeriod,cciPrice,i);

      recTP[i] = 0;
      recSL[i] = 0;

//+------------------------------------------------------------------+ Finding signals

      if(UseAllBars || barsLimit > limit) barsLimit = limit;

      if(i <= barsLimit)
      {
         bool doNotTrade = priceUp[i] > Up && priceDn[i] < Dn;

         if     (!doNotTrade && cci >  cciLevel && priceUp[i] > Up)    // Sell signals
         {
            sl = iOpen(NULL,0,i-1) + (stopLoss   / MathPow(10,Digits));
            tp = iOpen(NULL,0,i-1) - (takeProfit / MathPow(10,Digits));
            direction = -1;
         }
         
         else if(!doNotTrade && cci < -cciLevel && priceDn[i] < Dn)     // Buy signals
         {
            sl = iOpen(NULL,0,i-1) - (stopLoss   / MathPow(10,Digits));
            tp = iOpen(NULL,0,i-1) + (takeProfit / MathPow(10,Digits));
            direction =  1;
         }

//+------------------------------------------------------------------+ Finding if trade is a winner or loser

         if     (direction ==  1)
         {
            firstLossShift = CheckFirstLoserOrWinner(i,True, True, sl,tp);
            firstWinShift  = CheckFirstLoserOrWinner(i,True, False,sl,tp);
         }

         else if(direction == -1)
         {
            firstLossShift = CheckFirstLoserOrWinner(i,False,True, sl,tp);
            firstWinShift  = CheckFirstLoserOrWinner(i,False,False,sl,tp);
         }

         if     (firstWinShift > firstLossShift)
         {
            win = 1; loss = 0; balance =  takeProfit*Lots*TV;
         }

         else if(firstWinShift < firstLossShift)
         {
            win = 0; loss = 1; balance = -stopLoss  *Lots*TV;
         }

//+------------------------------------------------------------------+ Deleting trades when another trade was already open

         if(!AllowMultipleOpenTrades)
         {
            if(tp > 0) recTP[i] = tp; else if (recTP[i+1] > 0) recTP[i] = recTP[i+1];
            if(sl > 0) recSL[i] = sl; else if (recSL[i+1] > 0) recSL[i] = recSL[i+1];
   
            if((High[i] < recTP[i+1] && Low[i] > recSL[i+1]) || (High[i] < recSL[i+1] && Low[i] > recTP[i+1]))
            {
               recTP[i] = recTP[i+1];
               recSL[i] = recSL[i+1];
            }

            if(tp != recTP[i] || sl != recSL[i]) 
            {
               win = 0; loss = 0; balance = 0;
            }
         }

//+------------------------------------------------------------------+ Totals

         totalWins    += win;
         totalTrades  += (win + loss);
         totalBalance += balance;

         if(whichResult == "TotalTrades")  result = totalTrades;
         if(whichResult == "TotalWins")    result = totalWins;
         if(whichResult == "WinRate")
         {
            if(totalTrades == 0) result = 0;
            else                 result = (totalWins * 100) / totalTrades;
         }
         if(whichResult == "TotalBalance") result = totalBalance;

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+ Labels

   string Line3 = "Results coming from the Opti function:";
   string Line4 = "Trades: "+totalTrades+" / Wins: "+totalWins+" / Win%: "+DoubleToString((totalWins * 100) / totalTrades,2)+"% / Balance: "+totalBalance;

   color couleur = White;
   int   corner  = 0;
   int   x = 10;
   int   y = 40;

   DrawLabels(3,corner,x,y+30,Line3,couleur);
   DrawLabels(4,corner,x,y+40,Line4,couleur);

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+

      }
   }

//+------------------------------------------------------------------+ Freeing used arrays

   ArrayFree(recTP);
   ArrayFree(recSL);

//+------------------------------------------------------------------+

   return(result);
}

//+------------------------------------------------------------------+
//| CheckFirstLoserOrWinner                                          |
//+------------------------------------------------------------------+

int CheckFirstLoserOrWinner(int i, bool up, bool loss, double Sl, double Tp)
{
   int k,pre=0,result=0;

   for(k=i-1; k>=0; k--)
   {
      if(up)
      {
         if(loss) { if(High[k] >= Sl                                 && Low[k] <= Sl) { pre = k; if(pre > result) result = pre; } }
         else     { if(High[k] + (Spread / MathPow(10,Digits)) >= Tp && Low[k] <= Tp) { pre = k; if(pre > result) result = pre; } }
      }

      else
      {
         if(loss) { if(High[k] + (Spread / MathPow(10,Digits)) >= Sl && Low[k] <= Sl) { pre = k; if(pre > result) result = pre; } }
         else     { if(High[k] >= Tp && Low[k] + (Spread / MathPow(10,Digits)) <= Tp) { pre = k; if(pre > result) result = pre; } }
      }
   }

   return(result);
}

//+------------------------------------------------------------------+
//| Text labels                                                      |
//+------------------------------------------------------------------+

void DrawLabels(int instance, int corner, int x, int y, string text, color couleur)
{
   string name = "Labels_"+instance;
   ObjectCreate (name,OBJ_LABEL,0,0,0);
   ObjectSetText(name,text,7,"Verdana",couleur);
   ObjectSet(name,OBJPROP_CORNER,corner);
   ObjectSet(name,OBJPROP_XDISTANCE,x);
   ObjectSet(name,OBJPROP_YDISTANCE,y);
}

//+------------------------------------------------------------------+

void DeleteLabels()
{
   string lookFor       = "Labels_";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//+------------------------------------------------------------------+
//| The End                                                          |
//+------------------------------------------------------------------+